using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Emgu.CV;
using Emgu.CV.CvEnum;
using Emgu.CV.Structure;

namespace TestProject
{
    /// <summary>
    ///  
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  - MainForm()

        /// <summary>
        /// 
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.filePathTextBox.TextChanged += filePathTextBox_TextChanged;
            this.filePathButton.Click        += filePathButton_Click;

            this.filePathTextBox.Text = "sample.png";
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region   ؽƮ ڽ ؽƮ  óϱ - filePathTextBox_TextChanged(sender, e)

        /// <summary>
        ///   ؽƮ ڽ ؽƮ  óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void filePathTextBox_TextChanged(object sender, EventArgs e)
        {
            DetectShape();
        }

        #endregion
        #region   ư Ŭ óϱ - filePathButton_Click(sender, e)

        /// <summary>
        ///   ư Ŭ óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void filePathButton_Click(object sender, EventArgs e)
        {
            DialogResult result = this.openFileDialog.ShowDialog();

            if(result == DialogResult.OK || result == DialogResult.Yes)
            {
                this.filePathTextBox.Text = this.openFileDialog.FileName;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region  ϱ - DetectShape()

        /// <summary>
        ///  ϱ
        /// </summary>
        public void DetectShape()
        {
            if(this.filePathTextBox.Text != string.Empty)
            {
                StringBuilder stringBuilder = new StringBuilder(" : ");

                Image<Bgr, Byte> sourceImage = new Image<Bgr, byte>(this.filePathTextBox.Text).Resize(400, 400, INTER.CV_INTER_LINEAR, true);

                Image<Gray, Byte> grayscaleImage = sourceImage.Convert<Gray, Byte>().PyrDown().PyrUp();

                Stopwatch stopwatch = Stopwatch.StartNew();

                #region  Ѵ.

                double cannyThreshold             = 180.0;
                double circleAccumulatorThreshold = 120;

                CircleF[][] circleArray = grayscaleImage.HoughCircles
                (
                    new Gray(cannyThreshold),
                    new Gray(circleAccumulatorThreshold),
                    2.0,  //  ߽ ⿡ Ǵ  ػ
                    20.0, // ּ Ÿ
                    5,    // ּ ݰ
                    0     // ִ ݰ
                );

                #endregion

                stopwatch.Stop();
            
                stringBuilder.Append(String.Format(" - {0} и; ", stopwatch.ElapsedMilliseconds));

                stopwatch.Reset();
                
                stopwatch.Start();

                #region  Ѵ.
            
                double cannyThresholdLinking = 120.0;

                Image<Gray, Byte> cannyEdges = grayscaleImage.Canny(cannyThreshold, cannyThresholdLinking);

                LineSegment2D[][] lineArray = cannyEdges.HoughLinesBinary
                (
                    1,              // Ÿ ػ (ȼ  )
                    Math.PI / 45.0, //  Ǵ  ػ
                    20,             // Ѱ
                    30,             // ּ  ʺ
                    10              //  
                );

                #endregion

                stopwatch.Stop();

                stringBuilder.Append(String.Format(" - {0} и; ", stopwatch.ElapsedMilliseconds));

                stopwatch.Reset();
                
                stopwatch.Start();

                #region ﰢ 簢 Ѵ.

                List<Triangle2DF> triangleList = new List<Triangle2DF>();

                List<MCvBox2D> rectangleList = new List<MCvBox2D>();

                using(MemStorage storage = new MemStorage())
                {
                    for(Contour<Point> pointContour = cannyEdges.FindContours(CHAIN_APPROX_METHOD.CV_CHAIN_APPROX_SIMPLE, RETR_TYPE.CV_RETR_LIST, storage); pointContour != null; pointContour = pointContour.HNext)
                    {
                        Contour<Point> currentContour = pointContour.ApproxPoly(pointContour.Perimeter * 0.05, storage);

                        if(currentContour.Area > 250)
                        {
                            if(currentContour.Total == 3) // ﰢ 
                            {
                                Point[] pointArray = currentContour.ToArray();

                                triangleList.Add(new Triangle2DF(pointArray[0], pointArray[1], pointArray[2]));
                            }
                            else if(currentContour.Total == 4) // 簢 
                            {
                                #region    [80, 100] ̳ Ѵ.

                                bool isRectangle = true;

                                Point[] pointArray = currentContour.ToArray();

                                LineSegment2D[] edgeArray = PointCollection.PolyLine(pointArray, true);

                                for(int i = 0; i < edgeArray.Length; i++)
                                {
                                    double angle = Math.Abs(edgeArray[(i + 1) % edgeArray.Length].GetExteriorAngleDegree(edgeArray[i]));

                                    if(angle < 80 || angle > 100)
                                    {
                                        isRectangle = false;

                                        break;
                                    }
                                }

                                #endregion

                                if(isRectangle)
                                {
                                    rectangleList.Add(currentContour.GetMinAreaRect());
                                }
                            }
                        }
                    }
                }

                #endregion

                stopwatch.Stop();

                stringBuilder.Append(String.Format("ﰢ/簢 - {0} и; ", stopwatch.ElapsedMilliseconds));

                Text = stringBuilder.ToString();

                this.originalImageBox.Image = sourceImage;

                #region ﰢ/簢 ׸.

                Image<Bgr, Byte> triangleRectangleImage = sourceImage.CopyBlank();

                foreach(Triangle2DF triangle in triangleList)
                {
                    triangleRectangleImage.Draw(triangle, new Bgr(Color.DarkBlue), 2);
                }

                foreach(MCvBox2D rectangle in rectangleList)
                {
                    triangleRectangleImage.Draw(rectangle, new Bgr(Color.DarkOrange), 2);
                }

                this.triangleRectangleImageBox.Image = triangleRectangleImage;

                #endregion
                #region  ׸.

                Image<Bgr, Byte> circleImage = sourceImage.CopyBlank();

                foreach(CircleF circle in circleArray[0])
                {
                    circleImage.Draw(circle, new Bgr(Color.Brown), 2);
                }

                this.circleImageBox.Image = circleImage;

                #endregion
                #region  ׸.

                Image<Bgr, Byte> lineImage = sourceImage.CopyBlank();

                foreach(LineSegment2D line in lineArray[0])
                {
                    lineImage.Draw(line, new Bgr(Color.Green), 2);
                }

                this.lineImageBox.Image = lineImage;

                #endregion
            }
        }

        #endregion
    }
}